
--double brackets is just a multi-line string
local shader_code = [[
extern vec4[16] base_palette;
extern vec4[16] live_palette;

vec4 effect(vec4 colour, Image image, vec2 uvs, vec2 screen_coords) {
	vec4 pixel = Texel(image, uvs);
	for(int i = 0; i < 16; i++){
		vec4 colour = base_palette[i];
		if(pixel == colour)
			return live_palette[i];
	}
	return pixel;
}
]]
--Makes everything red
--return vec4(1.0, 0.0, 0.0, 1.0);

--Multiplies everythng by setColor, so you can colourise everything one colour
--return pixel * colour;

--Get the average of the 3 colours and just return the average for the 3 channels - makes everything black and white, but it's not very high contrast
--float av = (pixel.r + pixel.g + pixel.b) / 3.0;
--return vec4(av, av, av, 1.0);

local push = require "push"

local gameWidth, gameHeight = 64, 64 --fixed game resolution
local windowWidth, windowHeight = love.window.getDesktopDimensions()

local bg_music = love.audio.newSource("assets/test.mod", "stream")
local sfx = love.audio.newSource("assets/ohno.wav", "static")

-- The finished player object
player = {
	x = 16,
	y = 16,
	img = nil -- store the sprite we'll be drawing
}

enemy = {
	x=32,
	y=32,
	img = nil
}

enemy2 = {
	x=40,
	y=32,
	img = nil
}

base_palette = {}
live_palette = {}

function love.load()
	love.graphics.setDefaultFilter("nearest","nearest",1)
	push:setupScreen(gameWidth, gameHeight, windowWidth, windowHeight, {fullscreen = true, pixelperfect = true})

	player.img = love.graphics.newImage('assets/player.png')
	enemy.img = love.graphics.newImage('assets/player.png')
	enemy2.img = love.graphics.newImage('assets/player.png')

	love.audio.play(bg_music)

	--SHADER STUFF
	--load Pico8 style palette for shader
	local palette_img = love.image.newImageData('assets/palette.png')
	for x = 1, palette_img:getWidth() do
        local r,g,b,a = palette_img:getPixel(x - 1, 0)
        base_palette[#base_palette + 1] = {r,g,b,a}
	end
	live_palette = {unpack(base_palette)}--quick way to copy table
	shader = love.graphics.newShader(shader_code)
	shader:send("base_palette", unpack(base_palette))
	shader:send("live_palette", unpack(live_palette))
end

function love.update(dt)
	if love.keyboard.isDown("left") then
		player.x = player.x - 1
	end
	if love.keyboard.isDown("right") then
		player.x = player.x + 1
	end
	if love.keyboard.isDown("up") then
		--love.audio.play(sfx)
		player.y = player.y - 1
	end
	if love.keyboard.isDown("down") then
		player.y = player.y + 1
	end
end

function love.keypressed(key)
	if key == "escape" then
		love.event.push("quit")
	end

	if key == "up" then
		--This will produce a value that varies between 0.9 and 1.1.
		local rnd = (love.math.random() * 0.2) +0.9
		print(rnd)
		sfx:setPitch(rnd)
		sfx:play()
	end

	if key == "r" then 
		--LUA doesn't need brackets
		love.event.quit "restart"
	end
end

function love.draw(dt)
	--START LOW RES
	push:start()
	--set the 16 colour palette shader
	love.graphics.setShader(shader)
	--Draw some text in the low res window
	love.graphics.print("Hello World", 0, 0)
	love.graphics.print("Again-------", 0, 48)
	--Draw a sprite in the low res window
	love.graphics.draw(player.img, player.x, player.y)
	--affect the palette of one enemy
	pal(10,11)
	love.graphics.draw(enemy.img, enemy.x, enemy.y)
	pal_reset()
	love.graphics.draw(enemy2.img, enemy2.x, enemy2.y)
	--Clear the shader
	love.graphics.setShader()
	--end the low res stuff, reset to desktop res
	push:finish()
	--END LOW RES

	--record the state of the view coordinates
	love.graphics.push()
	--move the view coordinates to 100,100
	love.graphics.translate(100, 100)
	--Draw a wonky square depending on the time
	local t = love.timer.getTime()
	love.graphics.shear(math.cos(t), math.cos(t * 1.3))
	love.graphics.rectangle('fill', 0, 0, 100, 50)
	--Return the view coordinates to normal
	love.graphics.pop()
end

--source: index to change
--destination: index of colour to change it to
--[[ Uses the base pallete to get colours, so you don't
have to worry about the destination colour being 
different than you expect]]
function pal(s,d)
	live_palette[s] = base_palette[d]
	shader:send("live_palette", unpack(live_palette))
end
--quick version of above, doesn't send the whole array every time. 
--Use when making a number of changes, then finish with pal_update()
function pal_q(s,d)
	live_palette[s] = base_palette[d]
end
function pal_update()
	shader:send("live_palette", unpack(live_palette))
end
--resets pallete to default
function pal_reset()
	for i=1,#base_palette do
		live_palette[i]=base_palette[i]
	end
	shader:send("live_palette", unpack(live_palette))
end
--sets entire palette to the one colour, good for flashing a sprite when hit
function pal_flood(d)
	for i=1,#base_palette do
		live_palette[i]=base_palette[d]
	end
	shader:send("live_palette", unpack(live_palette))
end